/**
 * node-crc32-stream
 *
 * Copyright (c) 2014 Chris Talkington, contributors.
 * Licensed under the MIT license.
 * https://github.com/archiverjs/node-crc32-stream/blob/master/LICENSE-MIT
 */
exports = module.exports = require('./crc32-stream');
exports.CRC32Stream = exports;
exports.DeflateCRC32Stream = require('./deflate-crc32-stream');