module.exports = {
  'camelCase': require('./string/camelCase'),
  'capitalize': require('./string/capitalize'),
  'deburr': require('./string/deburr'),
  'endsWith': require('./string/endsWith'),
  'escape': require('./string/escape'),
  'escapeRegExp': require('./string/escapeRegExp'),
  'kebabCase': require('./string/kebabCase'),
  'pad': require('./string/pad'),
  'padLeft': require('./string/padLeft'),
  'padRight': require('./string/padRight'),
  'parseInt': require('./string/parseInt'),
  'repeat': require('./string/repeat'),
  'snakeCase': require('./string/snakeCase'),
  'startCase': require('./string/startCase'),
  'startsWith': require('./string/startsWith'),
  'template': require('./string/template'),
  'templateSettings': require('./string/templateSettings'),
  'trim': require('./string/trim'),
  'trimLeft': require('./string/trimLeft'),
  'trimRight': require('./string/trimRight'),
  'trunc': require('./string/trunc'),
  'unescape': require('./string/unescape'),
  'words': require('./string/words')
};
