module.exports = {
  'chain': require('./chain/chain'),
  'commit': require('./chain/commit'),
  'lodash': require('./chain/lodash'),
  'plant': require('./chain/plant'),
  'reverse': require('./chain/reverse'),
  'run': require('./chain/run'),
  'tap': require('./chain/tap'),
  'thru': require('./chain/thru'),
  'toJSON': require('./chain/toJSON'),
  'toString': require('./chain/toString'),
  'value': require('./chain/value'),
  'valueOf': require('./chain/valueOf'),
  'wrapperChain': require('./chain/wrapperChain')
};
